from flet.auth.providers.auth0_oauth_provider import Auth0OAuthProvider
from flet.auth.providers.azure_oauth_provider import AzureOAuthProvider
from flet.auth.providers.github_oauth_provider import GitHubOAuthProvider
from flet.auth.providers.google_oauth_provider import GoogleOAuthProvider
